#!/usr/bin/env python

r""" mglob - enhanced file list expansion module

Use as stand-alone utility (for xargs, `backticks` etc.), 
or a globbing library for own python programs. Globbing the sys.argv is something 
that almost every Windows script has to perform manually, and this module is here
to help with that task. Also Unix users will benefit from enhanced modes 
such as recursion, exclusion, directory omission...

Unlike glob.glob, directories are not included in the glob unless specified 
with 'dir:'

'expand' is the function to use in python programs. Typical use
to expand argv (esp. in windows)::

    try:
        import mglob 
        files = mglob.expand(sys.argv[1:])
    except ImportError:
        print "mglob not found; try 'easy_install mglob' for extra features"
        files = sys.argv[1:] 

Note that for unix, shell expands *normal* wildcards (*.cpp, etc.) in argv.
Therefore, you might want to use quotes with normal wildcards to prevent this 
expansion, in order for mglob to see the wildcards and get the wanted behaviour.
Not quoting the wildcards is harmless and typically has equivalent results, though.

Author: Ville Vainio <vivainio@gmail.com>
License: MIT Open Source license

"""

#Assigned in variable for "usage" printing convenience"

globsyntax = """\
This program allows specifying filenames with "mglob" mechanism.
Supported syntax in globs (wilcard matching patterns)::

 *.cpp ?ellowo*                
     - obvious. Differs from normal glob in that dirs are not included.
       Unix users might want to write this as: "*.cpp" "?ellowo*"
 rec:/usr/share=*.txt,*.doc    
     - get all *.txt and *.doc under /usr/share, 
       recursively
 rec:/usr/share
     - All files under /usr/share, recursively
 rec:*.py
     - All .py files under current working dir, recursively
 foo                           
     - File foo, or all files in dir foo
 !*.bak readme*                   
     - readme*, exclude files ending with .bak
 !.svn/ !.hg/ !*_Data/ rec:.
     - Skip .svn, .hg, foo_Data dirs (and their subdirs) in recurse.
       Trailing / is the key, \ does not work!
 dir:foo                       
     - the directory foo (not files in foo)
 dir:*                         
     - all directories in current folder
 foo.py bar.* !h* rec:*.py
     - Obvious. !h* exclusion only applies for rec:*.py.
       foo.py is *not* included twice.
 @filelist.txt
     - All files listed in 'filelist.txt' file, on separate lines.
 """


__version__ = "0.3"


import os,glob,fnmatch,sys
from sets import Set as set

                
def expand(flist):
    """ Expand the glob(s) in flist.
    
    flist may be either a whitespace-separated list of globs/files
    or an array of globs/files.
    
    """
    if isinstance(flist, basestring):
        flist = flist.split()
    done_set = set()
    denied_set = set()

    def recfind(p, pats = ["*"]):
        denied_dirs = ["*" + d+"*" for d in denied_set if d.endswith("/")]
        #print "de", denied_dirs
        for (dp,dnames,fnames) in os.walk(p):
            # see if we should ignore the whole directory
            dp_norm = dp.replace("\\","/") + "/"
            deny = False
            #print "dp",dp
            for deny_pat in denied_dirs:
                if fnmatch.fnmatch( dp_norm, deny_pat):
                    deny = True
                    break
            if deny:
                continue

                    
            for f in fnames:
                matched = False
                for p in pats:
                    if fnmatch.fnmatch(f,p):
                        matched = True
                        break
                if matched:
                    yield os.path.join(dp,f)            

    def once_filter(seq):
        for it in seq:
            p = os.path.abspath(it)
            if p in done_set:
                continue
            done_set.add(p)
            deny = False
            for deny_pat in denied_set:
                if fnmatch.fnmatch(os.path.basename(p), deny_pat):
                    deny = True
                    break
            if not deny:
                yield it
        return
            
    res = []

    for ent in flist:
        ent = os.path.expanduser(os.path.expandvars(ent))
        if ent.lower().startswith('rec:'):
            fields = ent[4:].split('=')            
            if len(fields) == 2:
                pth, patlist = fields
            elif len(fields) == 1:
                if os.path.isdir(fields[0]):
                    # single arg is dir
                    pth, patlist = fields[0], '*'
                else: 
                    # single arg is pattern
                    pth, patlist = '.', fields[0]
                    
            elif len(fields) == 0:
                pth, pathlist = '.','*'
                
            pats = patlist.split(',')
            res.extend(once_filter(recfind(pth, pats)))
        # filelist
        elif ent.startswith('@') and os.path.isfile(ent[1:]):
            res.extend(once_filter(open(ent[1:]).read().splitlines()))
        # exclusion
        elif ent.startswith('!'):
            denied_set.add(ent[1:])
        # glob only dirs
        elif ent.lower().startswith('dir:'):
            res.extend(once_filter(filter(os.path.isdir,glob.glob(ent[4:]))))
            
        # get all files in the specified dir
        elif os.path.isdir(ent):
            res.extend(once_filter(filter(os.path.isfile,glob.glob(ent + os.sep+"*"))))
            
        # glob only files

        elif '*' in ent or '?' in ent:
            res.extend(once_filter(filter(os.path.isfile,glob.glob(ent))))

        else:
            res.extend(once_filter([ent]))
    return res
            
            
def test():
    assert (
        expand("*.py ~/.ipython/*.py rec:/usr/share/doc-base") == 
        expand( ['*.py', '~/.ipython/*.py', 'rec:/usr/share/doc-base'] ) 
        )
    
def main():
    if len(sys.argv) < 2:
        print globsyntax
        return
    
    print "\n".join(expand(sys.argv[1:])),

def mglob_f(self, arg):
    from IPython.genutils import SList
    if arg.strip():
        return SList(expand(arg))
    print "Please specify pattern!"
    print globsyntax

def init_ipython(ip):
    """ register %mglob for IPython """
    mglob_f.__doc__ = globsyntax
    ip.expose_magic("mglob",mglob_f)  
    
# test()
if __name__ == "__main__":
    main()



#!/usr/bin/env python
""" ahcm - ad hoc configuration management tool

Usage:
    
    ahcm apply mypatch /prj/foo
    ahcm apply blah.txt foo.txt /prj/foo
    
    - Copy all files under directory mypatch over corresponding files in 
    /prj/foo. Do the same for blah.txt and foo.txt 
    If file doesn't exist under target dir, it's NOT created.
    
    ahcm today [places]
        
    - List all files changed today
    
    ahcm junk [places]
    
    - Remove all junk files (*~, *.bak...)
    
    
"""

# Copyright (c) Ville Vainio <vivainio@gmail.com>

# Licensed under the MIT license, see
#  http://www.opensource.org/licenses/mit-license.php

from path import path
import os,shutil,sys,time,fnmatch
import pickle
import mglob
import pprint

class DirLookup:
    def __init__(self,dir=None):
        self.filedict = {}
        if dir:
            self.pushfiles(dir)

    def pushfiles(self,dir):
        print "Pushing all in",dir
        i = 0
        #         for pathname,dirs,files in os.walk(dir):
        #             i+=1
        #             if i%20 == 0:
        #                 print path
        #                 print ".",
        #             pth = path(pathname)
        for f in path(dir).walkfiles():
            self.filedict[str(f.basename().lower())] = self.filedict.get(
                f.basename().lower(), []) + [f.abspath()]
            
    def lookup(self,f):
        return self.filedict[f.lower()]

    def exists(self,f):
        if self.filedict.has_key(f.lower()):
            return self.lookup(f)
        return None

    def allfiles(self):
        return self.filedict.items()


def getdiffs(old,new):
    diffs = os.popen("diff -c %s %s" % (old,new)).read()
    return diffs

_confirm_all = False

def confirm(prompt):
    global _confirm_all
    while 1:
        if not _confirm_all:
            r = raw_input(prompt)

        if _confirm_all or r == 'y' :
            return True
        elif r == 'a':
            _confirm_all = True
            return True
        elif r == 'n':
            return False
        
def similar_files(f1,f2):
    
    if f1.size != f2.size or f1.mtime != f2.mtime:
        return False
    if f1.text() != f2.text():
        return False
    return True

def find_good_match(pname, tries):
    pparts = pname.splitall()
    trieparts = [p.splitall() for p in tries]
    i = 0
    good = None

    while i < len(pparts):
        i+=1
        goodcount = 0
        for idx,trie in enumerate(trieparts):
            if trie[-i:] == pparts[-i:]:
                good = idx
                goodcount+=1
        if goodcount == 0:
            return None
        if goodcount == 1:
            print pname,"goodmatches",tries[good]
            return tries[good]
    return None

def expand_filelist(places):
    #return mglob.expand(places)
    pfiles = []
    for el in map(path,places):
        #print el
        if el.isdir():
            pfiles.extend(el.walkfiles())
        else:
            pfiles.append(el)
    return pfiles
    

def applypatch(*args): #patchdir, target):
    """ahcm apply [files...] target
    ahcm apply mypatch /prj/foo
    ahcm apply blah.txt foo.txt /prj/foo
    
    - Copy all files under directory mypatch over corresponding files in 
    /prj/foo. Do the same for blah.txt and foo.txt 
    If file doesn't exist under target dir, it's NOT created.
    """
    #print "Applying patch",patchdir,"to",target
    
    pfiles = expand_filelist(args[:-1])

    target = path(args[-1])
    
    lookup = DirLookup(target)
    #print lookup.allfiles()
    #patched = path(patchdir).
    #pdir = path(patchdir)
    #target= path(target)
    #pfiles = [pdir.relpathto(p) for p in pdir.walkfiles()]
        
    targets = []
    print pfiles
    for pf in pfiles:
        if not lookup.exists(pf.basename()):
            print "Ignoring:",pf
        else:
            lu = lookup.lookup(pf.basename())
            targets.append((pf, find_good_match(pf, lu)))
        
    print targets
    for src, tgt in targets:
        if tgt is None:
            print "Skipping, Multiple matches for",src
            continue
        print "%s -> TGT/%s" % (src, target.relpathto(tgt)),
        diffs = getdiffs(tgt,src)
        if not diffs:
            print "\t[unchanged]"
            continue
        if not _confirm_all:
            print diffs
        go = confirm("Copy (y/n/a): ")
        if go:
            print "Copying!"
            os.chmod(tgt, 0777) 
            shutil.copy2(src,tgt)

def changed_today(*places):
    """ahcm today [places]
        
    - List all files changed today
    """
    if not places:
        places = ['.']
    interval = 60 * 60 * 12 # 12 hours in seconds
    now = int(time.time())
    files = [(now-f.mtime, f) for f in expand_filelist(places) if now - f.mtime < interval]
    files.sort(reverse=True)
    def timestr(secs):
        hours = secs / 60
        mins = secs % 60
        if hours > 0:
            return "%d:%02d" % (hours, mins)
        return "%d min" % mins
    for t,f in files:
        print "%-40s %s" % (f, timestr(t/60))
    
def kill_junk(*places):
    """ahcm junk [places]
    
    - Remove all junk files (*~, *.bak...)
    """

    junk_pats = ['*~','*.bak','*.pyc']
    def isjunk(f):
        for pat in junk_pats:
            if fnmatch.fnmatch(f.basename(), pat):
                return True
            
    if not places:
        places = ['.']
    for f in expand_filelist(places):
        if isjunk(f):
            print f,'is junk'
            os.remove(f)
            
def lookup_files(*places):
    """ Quickly lookup files (a'la slocate) 
    
    ahcm lookup c:/my/dir - push all files in dir
    ahcm lookup blah - find a file with 'blah' in the name
    
    """
    filecache = 'c:/tmp/achm_filecache.pickle'
    if not places:
        places = ['.']
    if os.path.isfile(filecache):
        lookup = pickle.load(open(filecache))
    else:
        lookup = DirLookup()
    if os.path.isdir(places[0]):
        print "Pushing to lookup cache:",places[0]
        lookup.pushfiles(places[0])
        pickle.dump(lookup, open(filecache,'w'))
        return

    fpat = places[0]
    hits = lookup.exists(places[0])
    if hits:
        print "\n".join(hits)
    else:
        for k,v in lookup.allfiles():
            if fpat in k:
                print "\n".join(v)

def flatten_names(*args):    
    """ remove spaces and weird chars from file names """
    def flat_ch(ch):
        if not ord(ch) in range(128):
            return '_'
        if ch.isspace():
            return '_'
        return ch
    files = mglob.expand(args)
    pairs = [(path(f), path("".join(map(flat_ch, f)))) for f in files]
    changed = [(a,b) for (a,b) in pairs if a!=b]
    
    pprint.pprint(changed)
    if not raw_input('Enter y to commit:') == 'y':
        print "bailing out"
        return
    for old, new in changed:
        if not new.dirname().isdir():
            print "skipping",new," - dir does not exist"
        print "renaming",old
        old.rename(new)
    
    
def count_md5(*args):
    import md5
    #files = expand_filelist(args)
    files = [path(p) for p in mglob.expand(args)]
    print files
    out = "\n".join([md5.new(f.bytes()).hexdigest() + " " +f.basename() for f in files])
    print out
    return out


def rename_md5(*args):    
    """ Rename files by editing a text file with md5sums and file names
    
    First invocation of 'ahcm md5ren' will create the md5sums file, 
    edit it and invoke 'ahcm md5ren' again to commence with renames.
    """
    import md5
    fname = path('md5sums')
    if not fname.isfile():
        fname.write_text(count_md5(*(path('.').files())))
        print "Please edit file 'md5sums' and run again!"
        return
        
    md5f = fname.lines()
    ents = [l.rstrip().split(None,1) for l in md5f]
    realfiles = dict([(md5.new(f.bytes()).hexdigest(),f.basename()) for f in path('.').files()])
    print realfiles
    print ents
    for sum, newname in ents:
        oldname = realfiles.get(sum, None)
        if not oldname:
            print "skipping",newname
            continue
        if oldname == newname:
            print "Retain filename",oldname
            continue
        
        print "rename",oldname,'->',newname
        os.rename(oldname,newname)
        
        
    
        
def main():
    args = sys.argv[1:]
    cmd = args and args[0] or 'help'
    func = None
    
    commands = {
        'apply': applypatch,
        'today' : changed_today,
        'lookup' : lookup_files,        
        'junk' : kill_junk,
        'flatname' : flatten_names,
        'md5' : count_md5,
        'md5ren' : rename_md5  }
    
    if cmd == 'help':
        for c in commands:
            print '[%s]' % c
            print commands[c].__doc__
        return
    if cmd not in commands:
        print "Unknown command %s. Try one of help,%s instead." % (cmd, 
            ",".join(commands))
        return
    
    func = commands[cmd]
    
    apply(func, args[1:])
    
        

if __name__=="__main__":
    main()
